@@ -39,7 +39,7 @@ import univention.admin
 import univention.admin.handlers
 import univention.admin.localization
 from univention.admin.filter import conjunction, expression
-from univention.admin.handlers.dns import ARPA_IP4, ARPA_IP6
+from univention.admin.handlers.dns import ARPA_IP4, ARPA_IP6, DNSBase, is_dns  # noqa: F401
 from univention.admin.layout import Group, Tab
 
 
@@ -103,6 +103,7 @@ mapping.register('ptr_record', 'pTRRecord', encoding='ASCII')
 
 
 def ipv6(string):
+    # type: (str) -> str
     """
     >>> ipv6('0123456789abcdef0123456789abcdef')
     '0123:4567:89ab:cdef:0123:4567:89ab:cdef'
@@ -112,6 +113,7 @@ def ipv6(string):
 
 
 def calc_ip(rev, subnet):
+    # type: (str, str) -> ipaddress.IPv4Address | ipaddress.IPv6Address
     """
     >>> calc_ip(rev='8.0.0.0.7.0.0.0.6.0.0.0.5.0.0.0.4.0.0', subnet='0001:0002:0003:0').exploded
     '0001:0002:0003:0004:0005:0006:0007:0008'
@@ -122,16 +124,16 @@ def calc_ip(rev, subnet):
     parts.reverse()
     if ':' in subnet:
         string = ''.join(subnet.split(':') + parts)
-        ip = ipaddress.IPv6Address(u'%s' % (ipv6(string),))
+        return ipaddress.IPv6Address(u'%s' % (ipv6(string),))
     else:
         octets = subnet.split('.') + parts
         assert len(octets) == 4, octets
         addr = '.'.join(octets)
-        ip = ipaddress.IPv4Address(u'%s' % (addr,))
-    return ip
+        return ipaddress.IPv4Address(u'%s' % (addr,))
 
 
 def calc_rev(ip, subnet):
+    # type: (str, str) -> str
     """
     >>> calc_rev(ip='1.2.3.4', subnet='1.2')
     '4.3'
@@ -145,33 +147,39 @@ def calc_rev(ip, subnet):
         prefix = len(string)
         assert 1 <= prefix < 32
         string += '0' * (32 - prefix)
-        net = ipaddress.IPv6Network(u'%s/%d' % (ipv6(string), 4 * prefix), strict=False)
-        addr = ipaddress.IPv6Address(u'%s' % (ip,))
-        host = ''.join(addr.exploded.split(':'))
+        net6 = ipaddress.IPv6Network(u'%s/%d' % (ipv6(string), 4 * prefix), strict=False)
+        addr6 = ipaddress.IPv6Address(u'%s' % (ip,))
+        if addr6 not in net6:
+            raise ValueError()
+        host6 = ''.join(addr6.exploded.split(':'))
+        return '.'.join(reversed(host6[prefix:]))
     else:
         octets = subnet.split('.')
         prefix = len(octets)
         assert 1 <= prefix < 4
         octets += ['0'] * (4 - prefix)
-        net = ipaddress.IPv4Network(u'%s/%d' % ('.'.join(octets), 8 * prefix), strict=False)
-        addr = ipaddress.IPv4Address(u'%s' % (ip,))
-        host = addr.exploded.split('.')
-    if addr not in net:
-        raise ValueError()
-    return '.'.join(reversed(host[prefix:]))
+        net4 = ipaddress.IPv4Network(u'%s/%d' % ('.'.join(octets), 8 * prefix), strict=False)
+        addr4 = ipaddress.IPv4Address(u'%s' % (ip,))
+        if addr4 not in net4:
+            raise ValueError()
+        host4 = addr4.exploded.split('.')
+        return '.'.join(reversed(host4[prefix:]))
 
 
-class object(univention.admin.handlers.simpleLdap):
+class object(DNSBase):
     module = module
 
     def description(self):
+        # type: () -> str
         try:
-            return calc_ip(self.info['address'], self.superordinate.info['subnet']).compressed
+            if self.superordinate:
+                return calc_ip(self.info['address'] or '', self.superordinate.info['subnet'] or '').compressed
         except (LookupError, ValueError, AssertionError) as ex:
             log.warning('Failed to parse dn=%s: (%s)', self.dn, ex)
-            return super(object, self).description()
+        return super(object, self).description()
 
     def open(self):
+        # type: () -> None
         super(object, self).open()
         try:
             self.info['ip'] = calc_ip(self.info['address'], self.superordinate.info['subnet']).compressed
@@ -180,6 +188,7 @@ class object(univention.admin.handlers.simpleLdap):
             log.warning('Failed to parse dn=%s: (%s)', self.dn, ex)
 
     def ready(self):
+        # type: () -> None
         old_ip = self.oldinfo.get('ip')
         new_ip = self.info.get('ip')
         if old_ip != new_ip:
@@ -190,52 +199,24 @@ class object(univention.admin.handlers.simpleLdap):
                 raise univention.admin.uexceptions.InvalidDNS_Information(_('Reverse zone and IP address are incompatible.'))
         super(object, self).ready()
 
-    def _updateZone(self):
-        if self.update_zone:
-            self.superordinate.open()
-            self.superordinate.modify()
-
-    def __init__(self, co, lo, position, dn='', superordinate=None, attributes=[], update_zone=True):
-        self.update_zone = update_zone
-        univention.admin.handlers.simpleLdap.__init__(self, co, lo, position, dn, superordinate, attributes=attributes)
-
-    def _ldap_addlist(self):
-        return super(object, self)._ldap_addlist() + [
-            (self.superordinate.mapping.mapName('subnet'), self.superordinate.mapping.mapValue('subnet', self.superordinate['subnet'])),
-        ]
-
-    def _ldap_post_modify(self):
-        super(object, self)._ldap_post_modify()
-        if self.hasChanged(self.descriptions.keys()):
-            self._updateZone()
-
-    def _ldap_post_create(self):
-        super(object, self)._ldap_post_create()
-        self._updateZone()
-
-    def _ldap_post_remove(self):
-        super(object, self)._ldap_post_remove()
-        self._updateZone()
-
     @classmethod
     def lookup_filter_superordinate(cls, filter, superordinate):
-        filter.expressions.append(univention.admin.filter.expression('zoneName', superordinate.mapping.mapValueDecoded('subnet', superordinate['subnet']), escape=True))
+        # type: (univention.admin.filter.conjunction, univention.admin.handlers.simpleLdap) -> univention.admin.filter.conjunction
+        super(object, cls).lookup_filter_superordinate(filter, superordinate)
         filter = rewrite_rev(filter, superordinate.info['subnet'])
         return filter
 
     @classmethod
     def unmapped_lookup_filter(cls):
+        # type: () -> univention.admin.filter.conjunction
         return univention.admin.filter.conjunction('&', [
             univention.admin.filter.expression('objectClass', 'dNSZone'),
-            univention.admin.filter.conjunction('!', [univention.admin.filter.expression('relativeDomainName', '@')]),
-            univention.admin.filter.conjunction('|', [
-                univention.admin.filter.expression('zoneName', '*.in-addr.arpa', escape=False),
-                univention.admin.filter.expression('zoneName', '*.ip6.arpa', escape=False),
-            ]),
+            univention.admin.filter.expression('pTRRecord', '*', escape=False),
         ])
 
 
 def rewrite_rev(filter, subnet):
+    # type: (conjunction | expression, str) -> conjunction | expression
     """
     Rewrite LDAP filter expression and convert (ip) -> (zone,reversed)
 
@@ -269,13 +250,13 @@ def rewrite_rev(filter, subnet):
                 part if '*' in part else part.rjust(4, '0')[-4:]
                 for part in filter.value.split(':')
             )
-            suffix = '.ip6.arpa'
+            suffix = ARPA_IP6
         else:
             octets = subnet.split('.')
             prefix = len(octets)
             assert 1 <= prefix < 4
-            addr = filter.value.split('.')
-            suffix = '.in-addr.arpa'
+            addr = filter.value.split('.')  # type: ignore[assignment]
+            suffix = ARPA_IP4
         addr_net, addr_host = ('.'.join(reversed(_)) for _ in (addr[:prefix], addr[prefix:]))
         filter = conjunction('&', [
             expression('zoneName', addr_net + suffix),
@@ -289,8 +270,8 @@ lookup_filter = object.lookup_filter
 
 
 def identify(dn, attr):
-    return all([
-        b'dNSZone' in attr.get('objectClass', []),
-        b'@' not in attr.get('relativeDomainName', []),
-        (attr.get('zoneName', [b''])[0].decode('ASCII').endswith(ARPA_IP4) or attr.get('zoneName', [b''])[0].decode('ASCII').endswith(ARPA_IP6)),
-    ])
+    # type: (str, univention.admin.handlers._Attributes) -> bool
+    return bool(
+        attr.get('pTRRecord')
+        and is_dns(attr),
+    )
